/****************************************************************************
 *  
 * Project: Olimex AVR ISP with STK500 version 2 protocol support.
 *
 * Copyright by Olimex Ltd. All rights reserved.
 * 
 * File: board.h
 * Description: Board hardware properties.
 * Developer: Dimitar Dimitrov ( dinuxbg@gmail.com )
 *
 * Last change: $Date: 2007-12-13 17:33:07 +0200 (четвъртък, 13 Декември 2007) $
 * Revision: $Revision: 90 $
 * Id: $Id: board.h 90 2007-12-13 15:33:07Z dimitar $
 *
 ****************************************************************************/

#ifndef BOARD_H
#define BOARD_H

#if 1
//real board
#define DDR_SWSPI	DDRD
#define PORTR_SWSPI	PORTD
#define PINR_SWSPI	PIND
#define PIN_SWSPI_MISO	PD4
#define PIN_SWSPI_MOSI	PD3
#define PIN_SWSPI_SCK	PD5
#define PIN_SWSPI_CS	PD7
#define PIN_SWSPI_WP	PD6

#define DATAFLASH_PAGE_API	1

#define LED1_INIT()	(DDRC |= (1<<PC6))
#define LED1_OFF()	(PORTC &= ~(1<<PC6))
#define LED1_ON()	(PORTC |= (1<<PC6))
#define LED1_TOGGLE()	(PINC |= (1<<PC6)) /* only for AT90USB162 */

#define LED0_INIT()	(DDRC |= (1<<PC5))
#define LED0_OFF()	(PORTC &= ~(1<<PC5))
#define LED0_ON()	(PORTC |= (1<<PC5))
#define LED0_TOGGLE()	(PINC |= (1<<PC5)) /* only for AT90USB162 */


#define BUTTON1_INIT()		(DDRB &= ~(1<<PB6), PORTB |= (1<<PB6))
#define BUTTON1_PRESSED()	(~PINB & (1<<PB6))


/** Define I/O ports and pins for the ISP SPI port. */
#define DDR_SPI		DDRB
#define PORTR_SPI	PORTB
#define PINR_SPI	PINB
#define PIN_SS		PB0
#define PIN_MOSI	PB2
#define PIN_MISO	PB3
#define PIN_SCK		PB1
#define	DDR_RESET	DDRB
#define PORTR_RESET	PORTB
#define PIN_RESET	PB4

#define SPI_INVERT_OUT_DATA	0
#define SPI_INVERT_IN_DATA	1

#define DDR_OC2		DDRD
#define PORTR_OC2	PORTD
#define PIN_OC2		PD0

#define TLE_ON()	(PORTB &= ~(1<<PB5))
#define TLE_OFF()	(PORTB |= (1<<PB5))
#define TLE_INIT()	(DDRB |= (1<<PB5))

#else
//breadboard
#define DDR_SWSPI	DDRD
#define PORTR_SWSPI	PORTD
#define PINR_SWSPI	PIND
#define PIN_SWSPI_MISO	PD6
#define PIN_SWSPI_MOSI	PD5
#define PIN_SWSPI_SCK	PD3
#define PIN_SWSPI_CS	PD1
#define PIN_SWSPI_RST	PD2
//#define PIN_SWSPI_WP	PD6

#define DATAFLASH_PAGE_API	1

#define LED0_INIT()	(DDRD |= (1<<PD4))
#define LED0_ON()	(PORTD &= ~(1<<PD4))
#define LED0_OFF()	(PORTD |= (1<<PD4))

#define LED1_INIT()	(DDRD |= (1<<PD4))
#define LED1_OFF()	(PORTD &= ~(1<<PD4))
#define LED1_ON()	(PORTD |= (1<<PD4))

#define BUTTON1_INIT()		(DDRD &= ~(1<<PD7), PORTD |= (1<<PD7))
#define BUTTON1_PRESSED()	(~PIND & (1<<PD7))


/** Define I/O ports and pins for the ISP SPI port. */
#define DDR_SPI		DDRB
#define PORTR_SPI	PORTB
#define PINR_SPI	PINB
#define PIN_SS		PB0
#define PIN_MOSI	PB2
#define PIN_MISO	PB3
#define PIN_SCK		PB1
#define	DDR_RESET	DDRB
#define PORTR_RESET	PORTB
#define PIN_RESET	PB6

#define DDR_OC2		DDRC
#define PORTR_OC2	PORTC
#define PIN_OC2		PC6


#define TLE_ON()	(PORTB &= ~(1<<PB5))
#define TLE_OFF()	(PORTB |= (1<<PB5))
#define TLE_INIT()	(DDRB |= (1<<PB5))



#endif

#define aispcb_program_mem_page()	do{ LED0_TOGGLE(); LED1_TOGGLE();} while(0)
#define aispcb_compare_mem()		do{ LED0_TOGGLE(); LED1_ON();} while(0)


/** See whether the target VCC is above the configured threshold. */
#define TVCC_HIGH()	(!(ACSR & (1<<ACO)))

#define ispcb_enter_progmode()	\
	do { \
		TLE_OFF(); \
		DDRD |= (1<<PD0);	/* OC.0B */ \
		if(!TVCC_HIGH()) TLE_ON(); \
	}while(0)

#define ispcb_leave_progmode()		\
	do { \
		TLE_OFF(); \
		DDRD &= ~(1<<PD0);	/* OC.0B */ \
	} while(0)

/** SPI hardware configuration that will be loaded into the SPI Control
 * Register. */
#define SPCR_CONF	((1<<SPE) | (0<<DORD) | (1<<MSTR) | (0<<CPOL) | (0<<CPHA))

#define SPI_SET_CLOCK_DEFINITION()	\
static void spi_set_clock(uint8_t sckdur) { \
	if(sckdur == 0) { \
		spi_use_sw = 0; \
		/* freq=1843200, 3'b101 -> F_CPU/8 */ \
		SPCR = SPCR_CONF | (0<<SPR1)|(1<<SPR0); \
		SPSR = (1<<SPI2X); \
	} else if(sckdur == 1) { \
		spi_use_sw = 0; \
		/* freq=460800, 3'b110 -> F_CPU/32 */ \
		SPCR = SPCR_CONF | (1<<SPR1)|(0<<SPR0); \
		SPSR = (1<<SPI2X); \
	} else if(sckdur == 2) { \
		spi_use_sw = 0; \
		/* freq=115200, 3'b011 -> F_CPU/128 */ \
		SPCR = SPCR_CONF | (1<<SPR1)|(1<<SPR0); \
		SPSR = (0<<SPI2X); \
	} else { \
		spi_use_sw = sckdur; \
		SPCR = 0;	/* disable hardware SPI */ \
	} \
}

/** Enable 2x mode for SPI. */
//#define ISP_HWSPI_ENABLE_2X		1


/**
 * @brief Set external oscillator parameters.
 *
 * @param pscale Prescaler for Timer/Counter 1
 * @param cmatch Compare match value.
 *
 * @note Variable external target clock is not supported. Frequency is fixed
 * to F_CPU/256
 */
#define isp_set_extosc(pscale,cmatch) 	

extern void dbg_putchar(uint8_t c);

#endif /* BOARD_H */

