/****************************************************************************
 *  
 * Project: Olimex AVR ISP with MKII protocol support.
 *
 * Copyright by Olimex Ltd. All rights reserved.
 * 
 * File: main.c
 * Description: Main firmware logic goes here.
 * Developer: Dimitar Dimitrov ( dinuxbg __on__  gmail.com )
 *
 * Last change: $Date: 2007-12-13 17:33:07 +0200 (четвъртък, 13 Декември 2007) $
 * Revision: $Revision: 90 $
 * Id: $Id: main.c 90 2007-12-13 15:33:07Z dimitar $
 *
 ****************************************************************************/


#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <util/delay_basic.h>





#include "usb.h"
#include "massbulk.h"
//#include "board.h"
//#include "isp.h"
#include "msp430_mmc.h"
//#include "fat12_ro.h"
//#include "autoisp.h"


void delay_ms(uint8_t ms)
{
	while(ms) {
		--ms;
		_delay_ms(1.0);
	}
}



/**
 * Colour the STACK region in SRAM with a specific constant (see the source
 * below).
 *
 * Limitations: It does not colour the last byte (the first stack location).
 * It should not be an issue, though, as we're more interested in the far
 * bytes where we can collide with the static/global variables.
 */
void dbg_colour_stack (void) __attribute__ ((naked)) \
	__attribute__ ((section (".init1")));
void dbg_colour_stack (void)
{
	asm volatile (
		"ldi r28, lo8(_end)"		"\n\t"
		"ldi r29, hi8(_end)"		"\n\t"
		"ldi r31, hi8(__stack)"		"\n\t"
		"ldi r16, 0xD3"			"\n\t"
	   "1:" "st Y+, r16"			"\n\t"
		"cpi r28, lo8(__stack)"		"\n\t"
		"cpc r29, r31"			"\n\t"
		"brne 1b"			"\n\t"
		: :
	);
}

#if 0 
#define DBG_UART_TX_PORT	PORTC
#define DBG_UART_TX_DDR		DDRC
#define DBG_UART_BAUDRATE	9600
#define DBG_UART_TX_PIN		PC5


#define dbg_tx_init()	do { \
		DBG_UART_TX_PORT |= (1<<DBG_UART_TX_PIN); \
		DBG_UART_TX_DDR |= (1<<DBG_UART_TX_PIN); \
	} while(0)

/**
 * Software UART routine for transmitting debug information.
 *
 * Calculation for the number of CPU cycles, executed for one bit time:
 * F_CPU/9600 = (3*1+2) + (2*1 + (NDLY-1)*4 + 2+1) + 6*1
 *
 * (NDLY-1)*4 = F_CPU/9600 - 16
 * NDLY = (F_CPU/9600-16)/4+1
 */
#define DBG_UART_TX_NUM_DELAY_CYCLES	((F_CPU/DBG_UART_BAUDRATE-16)/4+1)
#define DBG_UART_TX_NUM_ADD_NOP		((F_CPU/DBG_UART_BAUDRATE-16)%4)
void dbg_putchar(uint8_t c)
{
	uint8_t sreg;
	uint16_t tmp;
	uint8_t numiter = 10;

	sreg = SREG;
	cli();

	asm volatile (
		/* put the START bit */
		"in %A0, %3"		"\n\t"	/* 1 */
		"cbr %A0, %4"		"\n\t"	/* 1 */
		"out %3, %A0"		"\n\t"	/* 1 */
		"nop"			"\n\t"	/* 1 */
		"nop"			"\n\t"	/* 1 */
		"nop"			"\n\t"	/* 1 */
		"nop"			"\n\t"	/* 1 */
		"nop"			"\n\t"	/* 1 */

		/* delay */
	   "1:" "ldi %A0, lo8(%5)"	"\n\t"	/* 1 */
		"ldi %B0, hi8(%5)"	"\n\t"	/* 1 */
	   "2:" "sbiw %A0, 1"		"\n\t"	/* 2 */
		"brne 2b"		"\n\t"	/* 1 if EQ, 2 if NEQ */
#if DBG_UART_TX_NUM_ADD_NOP > 0
		"nop"			"\n\t"	/* 1 */
  #if DBG_UART_TX_NUM_ADD_NOP > 1
		"nop"			"\n\t"	/* 1 */
    #if DBG_UART_TX_NUM_ADD_NOP > 2
		"nop"			"\n\t"	/* 1 */
    #endif
  #endif
#endif
		/* put data or stop bit */
		"in %A0, %3"		"\n\t"	/* 1 */
		"sbrc %1, 0"		"\n\t"	/* 1 if false,2 otherwise */
		"sbr %A0, %4"		"\n\t"	/* 1 */
		"sbrs %1, 0"		"\n\t"	/* 1 if false,2 otherwise */
		"cbr %A0, %4"		"\n\t"	/* 1 */
		"out %3, %A0"		"\n\t"	/* 1 */

		/* shift data, putting a stop bit at the empty location */
		"sec"			"\n\t"	/* 1 */
		"ror %1"		"\n\t"	/* 1 */

		/* loop 10 times */
		"dec %2"		"\n\t"	/* 1 */
		"brne 1b"		"\n\t"	/* 1 if EQ, 2 if NEQ */
		: "=&w" (tmp),			/* scratch register */
		  "=r" (c),			/* we modify the data byte */
		  "=r" (numiter)		/* we modify number of iter.*/
		: "I" (_SFR_IO_ADDR(DBG_UART_TX_PORT)),
		  "M" (1<<DBG_UART_TX_PIN),
		  "i" (DBG_UART_TX_NUM_DELAY_CYCLES),
		  "1" (c),			/* data */
		  "2" (numiter)
	);
	SREG = sreg;
}
#undef DBG_UART_TX_NUM_DELAY_CYCLES

#endif

/*-------------------------------------------------------------------------*/


void usbcb_wakeup(void)
{
	ACSR &= ~(1<<ACD);
}

void usbcb_suspend(void)
{

}

#define	STANDBY_PIN	PD6
#define	WAKE_PIN	PD4


/*void enable_wakeup(void)
{
	EICRB = 0;
	EIMSK |= 1<<5;
}

void sw_rst(void)
{	
	cli();
	wdt_enable(WDTO_2S);
	while(1);
}


ISR(INT5_vect)
{
	while (!(PIND&0b00010000));
	sw_rst();
}*/

void usb_stby(void)
{
	usb_current_configuration = 0;
	/* warning - why don't we enable the following line?
	 * usb_flags.pwr_on_rst = 0; */

	/* If we have power then there is no need for suspend. */
	if(usb_is_self_powered()) return;

	/* Clear Suspend Bit - done by the USB GEN ISR */

	/* Be sure to have interrupts enabled (WAKEUPE) to exit sleep mode */
	usb_inten_wakeup();
	//enable_wakeup();
	usb_intclr_wakeup();
	usb_intdis_suspend();

	/* Set USB suspend clock */
	usb_freeze_clock();

	/* Disable PLL */
	usb_pll_stop();

	usbcb_suspend();

	/* Put the MCU in sleep mode */
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	sleep_enable();
	sei();	/* we do need that in order to wakeup */
	sleep_cpu();
	sleep_disable();
	cli();

	usbcb_wakeup();

	/* Enable PLL */
	usb_pll_start();
	/* Wait PLL lock 
	 * @warning We wait a lot of time here! */
	while(!usb_pll_locked());
	/* Clear USB suspend clock */
	usb_resume_clock();
	/* Disable further interrupts on USB pad change. */
	usb_intclr_wakeup();
	usb_intdis_wakeup();
	/* Be prepared for other suspends. */
	usb_inten_suspend();

}

int main(void) __attribute__ ((naked)) ;
int main(void)
{
	//DDRC |= (1<<PC7); PORTC |= (1<<PC7);
	//wdt_disable();
/*	__asm__ __volatile__ (  \
    "in __tmp_reg__, __SREG__" "\n\t" \
    "cli" "\n\t" \
    "sts %0, %1" "\n\t" \
    "sts %0, __zero_reg__" "\n\t" \
    "out __SREG__,__tmp_reg__" "\n\t" \
    : "M" (_SFR_MEM_ADDR(_WD_CONTROL_REG)), \
    "r" ((uint8_t)(_BV(_WD_CHANGE_BIT) | _BV(WDE))) \
    : "r0" \
);*/
/*	asm("push r30");	// using GCC without optimizations results in
	asm("ldi r30, 0x18");	// a code not meeting the requirements
	asm("sts 0x60, r30");	// of the datasheet.
	asm("ldi r30, 0x7");
	asm("sts 0x60, r30");
	asm("pop r30");
	//WDTCR = (1<<WDCE) | (1<<WDE);
	//WDTCR = 0x07;					//increase Watchdog time-out period (Bit0,1,2 = 1)*/


	DDRB  |= (1<<7) | (1<<4);
	PORTB |= (1<<7) | (1<<4);

	DDRC  |= (1<<7) | (1<<6) | (1<<5) | (1<<2); // unused as outputs 
	PORTC |= (1<<7) | (1<<6) | (1<<5) | (1<<2);// in 1

	DDRD  |= (1<<5) | (1<<1) | (1<<0);
	PORTD |= (1<<5) | (1<<1) | (1<<0);


	//while (1);


	//if (!(PIND&0b01000000)) usb_stby();

	if (!(PINC&1<<4) )
	{
		cli(); //we will be woken up by the hardware reset
		set_sleep_mode(SLEEP_MODE_PWR_DOWN);
		sleep_enable();
		//sei();	/* we do need that in order to wakeup */
		sleep_cpu();

		__asm("NOP");
	}
	
	while (!spi_mmc_mem_init());	// we need the card initialized

	//enable_wakeup();
	//usb_stby();

	sei();

	//while (1);
	
	while(1) {
		usb_std_poll();
		ms_poll();
		//if (!(PIND&0b01000000)) usb_stby();
		if (!(PINC&1<<4) )
		{	
			cli(); //we will be woken up by the hardware reset
			set_sleep_mode(SLEEP_MODE_PWR_DOWN);
			sleep_enable();
			//sei();	/* we do need that in order to wakeup */
			sleep_cpu();

			__asm("NOP");
		}
	}
	return 1;
}
