/****************************************************************************
 *  
 * Project: Olimex AVR ISP with MKII protocol support.
 *
 * Copyright by Olimex Ltd. All rights reserved.
 * 
 * File: usb_descr.c
 * Description: USB descriptor definitions.
 * Developer: Dimitar Dimitrov ( dinuxbg __on__ gmail.com )
 *
 * Last change: $Date: 2007-11-26 17:39:42 +0200 (понеделник, 26 Ноември 2007) $
 * Revision: $Revision: 62 $
 * Id: $Id: usb_descr.c 62 2007-11-26 15:39:42Z dimitar $
 *
 ****************************************************************************/

#include <stdint.h>
#include <avr/pgmspace.h>

#include "usb_core.h"
#include "usb_conf.h"


struct usb_device_descriptor usb_device_descriptor PROGMEM = {
	sizeof(usb_device_descriptor),
	USB_DESCRTYPE_DEVICE,
	USB_REL_20,
	USB_DEVCLASS_NONE,
	USB_DEVSUBCLASS_NONE,
	USB_DEVPROT_NONE,
	USB_CTRLEP_MAX_PACKET_SIZE,
	USB_VENDOR_ID,
	USB_PRODUCT_ID,
	USB_DEV_RELNUM,
	1,			/* manufacturer string descriptor ID */
	2,			/* product string descriptor ID */
	3,			/* serial number string descriptor ID */
	1			/* number of configurations (we support only 
				 * single configurations for now) */
};

USB_CREATE_STRING_DESCRIPTOR(
		str_zero, 
		1, 
		0x0409
);

USB_CREATE_STRING_DESCRIPTOR(
		str_manuf, 
		11, 
		'O','l','i','m','e','x',' ','L','t','d','.'
);

USB_CREATE_STRING_DESCRIPTOR(
		str_prod, 
		19,
		'O','l','i','m','e','x',' ','A','V','R','I','S','P',' ','D','r','i','v','e'
);

USB_CREATE_STRING_DESCRIPTOR(
		str_sernum, 
		13, 
		'0','1','2','3','4','5','6','7','8','9','A','B','C'
);

struct usb_string_descriptor *usb_strings_table[] PROGMEM = {
	(struct usb_string_descriptor*) &str_zero,
	(struct usb_string_descriptor*) &str_manuf,
	(struct usb_string_descriptor*) &str_prod,
	(struct usb_string_descriptor*) &str_sernum
};

uint8_t usb_get_strtbl_size(void)
{
	return sizeof(usb_strings_table)/sizeof(usb_strings_table[0]);
}

/** User has to instantiate the configuration block structure. */
struct usb_configuration_block usb_configuration_block PROGMEM = {
	/* Configuration descriptor */
	{
		sizeof(struct usb_configuration_descriptor),
		USB_DESCRTYPE_CONFIGURATION,
		sizeof(usb_configuration_block),
		1,		/* one supported interface */
		1,		/* configuration value */
		0,		/* no configuration string */
		{ 0, 0, 0, 1 },	/* bus-powered device */
		USB_POWER_mA_TO_USBUNITS(USB_NEEDED_POWER_MA)

	},
	/* Mass Storage Interface descriptor */
	{
		sizeof(struct usb_interface_descriptor),
		USB_DESCRTYPE_INTERFACE,
		0,		/* interface number */
		0,		/* no alternate setting */
		2,		/* number of endpoints */
		USB_INTRFCLASS_MASS_STORAGE,
		USB_INTRFSUBCLASS_SCSI_TRANSPARENT,
		USB_INTRFPROT_BULK_ONLY,
		0		/* no interface string descriptor */
	},
	/* Data Endpoint IN descriptor */
	{
		sizeof(struct usb_endpoint_descriptor),
		USB_DESCRTYPE_ENDPOINT,
		{ USB_EPN_MS_IN, 0, USB_EPDIR_IN },	
		{ USB_EPTYPE_BULK, 0, 0, 0 },
		USB_EP_MS_IN_MAX_PACKET_SIZE,
		0x0A		/* interval, ignored for Bulk endpoints */
	},
	/* Data Endpoint OUT descriptor */
	{
		sizeof(struct usb_endpoint_descriptor),
		USB_DESCRTYPE_ENDPOINT,
		{ USB_EPN_MS_OUT, 0, USB_EPDIR_OUT },
		{ USB_EPTYPE_BULK, 0, 0, 0 },
		USB_EP_MS_OUT_MAX_PACKET_SIZE,
		0x0A		/* interval, ignored for Bulk endpoints */
	}
};


