/* The initial idea of handling everything here will probably not work, because 
 * the MMC is quite picky about things such as when the CS should go high, etc.
 * So instead we just retransmit the work to the other controller and let it
 * handle most of the work
 * The protocol is request->read state style
 * E.g. we send the command to be executed together with the parameters, then
 * periodically poll the other side to see the status of the command
 */

#include <stdint.h>
#include "msp430_mmc.h"                     /* our definitions :) */
#include "hwspi.h"

static uint32_t 	mmc_disk_size = 0;
static uint16_t		block_read_ptr = 0;
static uint32_t		tempaddress	= 0;
static uint8_t 		tempdata;


void			mmc_notimplemented()
{
	block_read_ptr++;
	block_read_ptr--;
}


uint8_t 	msp430_mmc_waitresp(void)
{

	uint8_t resp = MMC_RESP_BUSY;
	uint8_t timectr;
	while (resp == MMC_RESP_BUSY)
	{
		ResetSPI();
		stupiddelay();
		stupiddelay();
		stupiddelay();
		stupiddelay();
		stupiddelay();
		stupiddelay();
		spiTransfer(MMC_CMD_STATUS);
		stupiddelay();
		stupiddelay();
		stupiddelay();
		stupiddelay();
		stupiddelay();
		stupiddelay();
		stupiddelay();
		resp = spiTransfer(MMC_CMD_DUMMY);
		
		// the transmit buffer may be in an indetermined state
		if ((resp != MMC_RESP_OK) & (resp != MMC_RESP_FAIL)) resp = MMC_RESP_BUSY;
		for (timectr = resp_timeout; timectr; timectr--);
	}
	return resp;
}

uint32_t 	msp430_mmc_getsize()
{
	uint32_t size1, size2, size3, size4;
	if (mmc_disk_size != 0) return mmc_disk_size;
	mmc_disk_size = 0;
	ResetSPI();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	spiTransfer(MMC_CMD_SIZE);
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	size4 = spiTransfer(MMC_CMD_DUMMY);
	size3 = spiTransfer(MMC_CMD_DUMMY);
	size2 = spiTransfer(MMC_CMD_DUMMY);
	size1 = spiTransfer(MMC_CMD_DUMMY);
	
	mmc_disk_size = size4<<24;
	mmc_disk_size |= size3<<16;
	mmc_disk_size |= size2<<8;
	mmc_disk_size |= size1;
	
	return mmc_disk_size;
}

uint8_t	  msp430_mmc_mem_init(void)
{
	//msp430_mmc_init_spi();
	initSPI();
	
	ResetSPI();

	spiTransfer(MMC_CMD_STATUS);
	spiTransfer(MMC_CMD_STATUS);
	spiTransfer(MMC_CMD_INIT);
	

	if (msp430_mmc_waitresp()|MMC_RESP_OK) 
		{
			msp430_mmc_getsize();
			return 0;
		}
	return 1;
}

void			mmc_contread_start(uint32_t addr)
{	
	uint8_t add1, add2, add3, add4;
	
	
	//if (block_read_ptr) return; // should never do that !!!
	
	//if (addr>mmc_disk_size) return; // also...
	
	ResetSPI();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();


	spiTransfer(MMC_CMD_READ);
	//if (msp430_mmc_waitresp()!=MMC_RESP_OK) return MMC_RESP_FAIL; // strange...
	
	add1 = addr & 0xff;
	add2 = (addr >> 8) & 0xff;
	add3 = (addr >> 16) & 0xff;
	add4 = (addr >> 24) & 0xff;
	tempaddress = addr;
	spiTransfer(add4);
	stupiddelay();
	stupiddelay();
	stupiddelay();
	spiTransfer(add3);
	stupiddelay();
	stupiddelay();
	stupiddelay();
	spiTransfer(add2);
	stupiddelay();
	stupiddelay();
	stupiddelay();
	spiTransfer(add1);
	stupiddelay();
	stupiddelay();
	stupiddelay();
	
	ResetSPI();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();


	msp430_mmc_waitresp();//!=MMC_RESP_OK) return MMC_RESP_FAIL; // also strange
	
	spiTransfer(MMC_CMD_READ_BUFFER);
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	
	
}

uint8_t 	mmc_contread_fetch(void)
{
	
	if (block_read_ptr>=SECTOR_SIZE) 
	{
		tempaddress++;
		mmc_contread_start(tempaddress);
		block_read_ptr = 0;		
	}
	block_read_ptr++;
	tempdata = spiTransfer(MMC_CMD_DUMMY);
	return tempdata;
	//return spiTransfer(MMC_CMD_DUMMY);
	
}

void 			mmc_contread_stop(void)
{
	while (block_read_ptr<SECTOR_SIZE)
		{
			mmc_contread_fetch();
			block_read_ptr++;
		}
	block_read_ptr = 0;

}
