/* The initial idea of handling everything here will probably not work, because 
 * the MMC is quite picky about things such as when the CS should go high, etc.
 * So instead we just retransmit the work to the other controller and let it
 * handle most of the work
 * The protocol is request->read state style
 * E.g. we send the command to be executed together with the parameters, then
 * periodically poll the other side to see the status of the command
 */

#include <stdint.h>
#include "spi_mmc.h"                     /* our definitions :) */
#include "hwspi.h"
#include <avr/io.h>

static uint32_t 	mmc_disk_size = 0x400000;
//static uint32_t 	mmc_disk_size = 0x400;
static uint16_t		block_read_ptr = 0;
static uint32_t		tempaddress	= 0;
static uint8_t 		tempdata;

static uint8_t		startresponse=0;

#define BUFFER_SIZE 384
static uint8_t buffer[BUFFER_SIZE];
static uint16_t buffptr=0;


#define TRUE    1
#define FALSE   0


void			mmc_notimplemented()
{
	block_read_ptr++;
	block_read_ptr--;
}

uint8_t cardResponse(unsigned char expRes)
{
	unsigned char actRes;
	unsigned int count = 256;
	actRes = 0xFF;
	while ((actRes != expRes) && (count > 0))
	{
		actRes = spiTransfer(0xFF);	
		count--;
	}
	if (count == 0)
		return FALSE; 
	return TRUE;
}


uint32_t 	spi_mmc_getsize()
{
	
	
/*	mmc_disk_size = size4<<24;
	mmc_disk_size |= size3<<16;
	mmc_disk_size |= size2<<8;
	mmc_disk_size |= size1;*/
	
	return mmc_disk_size;
}

void sendCommand(unsigned char command, unsigned char argA,
                 unsigned char argB, unsigned char argC,
                 unsigned char argD, unsigned char CRC)
{	
	spiTransfer(command);
	spiTransfer(argA);
	spiTransfer(argB);
	spiTransfer(argC);
	spiTransfer(argD);
	spiTransfer(CRC);
}

uint8_t setBLockLength(void)
{
	MEM_PORT_OUT &= ~CS;
	sendCommand(CMD16,0,0,2,0,0xFF);
	if(cardResponse(0x00))
	{
		spiTransfer(0xFF);
		MEM_PORT_OUT |= CS;
		return TRUE;
	}
	spiTransfer(0xFF);
	MEM_PORT_OUT |= CS;
	return FALSE;
}

uint8_t	  spi_mmc_mem_init(void)
{
	//DDRC-->DDRB
	DDRC |= 1<<CS; // chip select output

	initSPI();
	
	ResetSPI();

	unsigned int count;
	MEM_PORT_OUT |= CS;	
	for (count = 0;count < 10; count++)
    {
		spiTransfer(0xFF);
    }
	MEM_PORT_OUT &= ~CS;
	sendCommand(CMD0,0,0,0,0,0x95);	// initializes the MMC
	if(cardResponse(0x01))
	{				
		count = 255;
		do
		{
			sendCommand(CMD1,0,0,0,0,0xFF);	// brings card out of idle state
			count--;
		} while ((!cardResponse(0x00)) && (count > 0));
		MEM_PORT_OUT |= CS;
		spiTransfer(0xFF);	
		if(count == 0)
			return FALSE;
	}
	else
	{
		MEM_PORT_OUT |= CS;
		return FALSE;		
	}
	//U1BR0 = 2;  //increase the spi speed
	//HALayerInit_HighSpeed();
	
	//setBLockLength();
	
	return TRUE;
}

void			mmc_contread_start(uint32_t addr)
{	
	uint8_t resp;
	//sector += sectorZero;
	tempaddress = addr;
	addr *= 2;	
	MEM_PORT_OUT &= ~CS;
	sendCommand(CMD17,(addr>>16)&0xFF,(addr>>8)&0xFF,addr&0xFF,0,0xFF);
	//if(cardResponse(0x00))
	//{
	//	if(resp(0xFE)) return;
	//}

	/*do
		resp=sd_Resp8b(iface); 
	while(firstblock==0xff && fb_timeout--);*/
	startresponse = cardResponse(0x00);
	//while (cardResponse(0xFE)==FALSE);
	startresponse |= (cardResponse(0xFE)*2);



	/*stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();*/
	
	//MEM_PORT_OUT |= CS;
	//spiTransfer(0xFF);	
}

uint8_t 	mmc_contread_fetch(void)
{

	/*tempdata = spiTransfer(0xff);
	return tempdata;*/
	if (block_read_ptr>=SECTOR_SIZE) 
	{
		tempaddress++;

		block_read_ptr = 0;	
		spiTransfer(0xFF);
		spiTransfer(0xFF);
		MEM_PORT_OUT |= CS;
		spiTransfer(0xFF);

		stupiddelay();
		stupiddelay();
		stupiddelay();
		stupiddelay();	
		stupiddelay();
		stupiddelay();
		stupiddelay();
		stupiddelay();



		mmc_contread_start(tempaddress);
		
		/*stupiddelay();
		stupiddelay();
		stupiddelay();
		stupiddelay();	
		stupiddelay();
		stupiddelay();
		stupiddelay();
		stupiddelay();*/
	}
	block_read_ptr++;
	tempdata = spiTransfer(0xff);
	buffer[buffptr] = tempdata;
	if (buffptr==BUFFER_SIZE) buffptr=0;
	return tempdata;
	
}

void 			mmc_contread_stop(void)
{
	/*while (block_read_ptr<SECTOR_SIZE)
		{
			mmc_contread_fetch();
			block_read_ptr++;
		}*/
	block_read_ptr = 0;
	spiTransfer(0xFF);
	spiTransfer(0xFF);
	MEM_PORT_OUT |= CS;
	spiTransfer(0xFF);
	/*stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();
	stupiddelay();*/

}
